@@ -30,6 +30,8 @@
  * SUCH DAMAGE.
  */
 #include "sh.h"
+#include "ed.h"
+#include "tw.h"
 
 /*
  * C shell
@@ -64,8 +66,10 @@ static Char *dolp;		/* Remaining chars from this word */
 static Char **dolnxt;		/* Further words */
 static int dolcnt;		/* Count of further words */
 static struct Strbuf dolmod; /* = Strbuf_INIT; : modifier characters */
-static int dolmcnt;		/* :gx -> INT_MAX, else 1 */
-static int dol_flag_a;		/* :ax -> 1, else 0 */
+
+static int ndolflags;		/* keep track of mod counts for each modifier */
+static int *dolmcnts;		/* :gx -> INT_MAX, else 1 */
+static int *dolaflags;		/* :ax -> 1, else 0 */
 
 static	Char	 **Dfix2	(Char *const *);
 static	int 	 Dpack		(struct Strbuf *);
@@ -352,7 +356,7 @@ quotspec:
 }
 
 static Char *nulvec[] = { NULL };
-static struct varent nulargv = {nulvec, STRargv, VAR_READWRITE, 
+static struct varent nulargv = {nulvec, STRargv, VAR_READWRITE,
 				{ NULL, NULL, NULL }, 0 };
 
 static void
@@ -378,12 +382,44 @@ Dgetdol(void)
     static Char *dolbang = NULL;
 
     cleanup_push(name, Strbuf_free);
-    dolmod.len = dolmcnt = dol_flag_a = 0;
+    dolmod.len = ndolflags = 0;
     c = sc = DgetC(0);
     if (c == DEOF) {
       stderror(ERR_SYNTAX);
       return;
     }
+    if ((c & TRIM) == '\'') {
+	const Char *cp;
+	struct Strbuf *expanded = Strbuf_alloc();
+
+	cleanup_push(expanded, Strbuf_free);
+	for (;;) {
+	    c = DgetC(0);
+	    if ((c & TRIM) == '\'')
+		break;
+	    if ((c & TRIM) == '\\') {
+		Strbuf_append1(name, (Char) c);
+		c = DgetC(0);
+	    }
+	    if (c == '\n' || c == DEOF) {
+		cleanup_until(name);
+		stderror(ERR_MISSING, '\'');
+		return;
+	    }
+	    Strbuf_append1(name, (Char) c);
+	}
+	Strbuf_terminate(name);
+	for (cp = name->s; (c = *cp) != 0; cp++) {
+	    if (c == '\\' && (c = parseescape(&cp, TRUE)) == CHAR_ERR)
+		c = '\\';
+	    Strbuf_append1(expanded, (Char) c | QUOTE);
+	}
+	Strbuf_terminate(expanded);
+	np = Strsave(expanded->s);
+	cleanup_until(name);
+	addla(np);
+	return;
+    }
     if (c == '{')
 	c = DgetC(0);		/* sc is { to take } later */
     if ((c & TRIM) == '#')
@@ -414,8 +450,19 @@ Dgetdol(void)
     case '<'|QUOTE: {
 	static struct Strbuf wbuf; /* = Strbuf_INIT; */
 
-	if (bitset)
-	    stderror(ERR_NOTALLOWED, "$?<");
+	if (bitset) {
+#ifdef FIONREAD
+	    int chrs;
+
+	    if (ioctl(OLDSTD, FIONREAD, (ioctl_t) &chrs) == -1)
+		chrs = 0;
+	    setDolp(putn((tcsh_number_t) chrs));
+	    cleanup_until(name);
+	    goto eatbrac;
+#else
+	    stderror(ERR_UNAVAILABLE, "$?<");
+#endif
+	}
 	if (dimen)
 	    stderror(ERR_NOTALLOWED, "$#<");
 	if (length)
@@ -718,26 +765,33 @@ fixDolMod(void)
 
     c = DgetC(0);
     if (c == ':') {
+	ndolflags = 0;
 	do {
-	    c = DgetC(0), dolmcnt = 1, dol_flag_a = 0;
+	    ++ndolflags;
+	    dolmcnts = xrealloc(dolmcnts, ndolflags * sizeof(int));
+	    dolaflags = xrealloc(dolaflags, ndolflags * sizeof(int));
+	    c = DgetC(0), dolmcnts[ndolflags - 1] = 1, dolaflags[ndolflags - 1] = 0;
 	    if (c == 'g' || c == 'a') {
-		if (c == 'g')
-		    dolmcnt = INT_MAX;
-		else
-		    dol_flag_a = 1;
+		if (c == 'g') {
+		    dolmcnts[ndolflags - 1] = INT_MAX;
+		} else {
+		    dolaflags[ndolflags - 1] = 1;
+		}
 		c = DgetC(0);
 	    }
-	    if ((c == 'g' && dolmcnt != INT_MAX) || 
-		(c == 'a' && dol_flag_a == 0)) {
-		if (c == 'g')
-		    dolmcnt = INT_MAX;
-		else
-		    dol_flag_a = 1;
+	    if ((c == 'g' && dolmcnts[ndolflags - 1] != INT_MAX) ||
+		(c == 'a' && dolaflags[ndolflags - 1] == 0)) {
+		if (c == 'g') {
+		    dolmcnts[ndolflags - 1] = INT_MAX;
+		} else {
+		    dolaflags[ndolflags - 1] = 1;
+		}
 		c = DgetC(0);
 	    }
 
 	    if (c == 's') {	/* [eichin:19910926.0755EST] */
 		int delimcnt = 2;
+		int esc = 0;
 		eChar delim = DgetC(0);
 		Strbuf_append1(&dolmod, (Char) c);
 		Strbuf_append1(&dolmod, (Char) delim);
@@ -746,23 +800,29 @@ fixDolMod(void)
 		    || Isdigit(delim) || any(" \t\n", delim)) {
 		    seterror(ERR_BADSUBST);
 		    break;
-		}	
-		while ((c = DgetC(0)) != DEOF) {
-		    Strbuf_append1(&dolmod, (Char) c);
-		    if(c == delim) delimcnt--;
-		    if(!delimcnt) break;
 		}
-		if(delimcnt) {
+		while ((c = DgetC(0)) != DEOF) {
+		    if (esc == 0 && c == '\\') {
+			esc = 1;
+			continue;
+		    }
+		    Strbuf_append1(&dolmod, (Char) c);
+		    if (!esc && c == delim) delimcnt--;
+		    if (!delimcnt) break;
+		    esc = 0;
+		}
+		if (delimcnt) {
 		    seterror(ERR_BADSUBST);
 		    break;
 		}
 		continue;
 	    }
-	    if (!any("luhtrqxes", c))
+	    if (!any(TCSH_MODIFIERS, c))
 		stderror(ERR_BADMOD, (int)c);
 	    Strbuf_append1(&dolmod, (Char) c);
-	    if (c == 'q')
-		dolmcnt = INT_MAX;
+	    if (c == 'q') {
+		dolmcnts[ndolflags - 1] = INT_MAX;
+	    }
 	}
 	while ((c = DgetC(0)) == ':');
 	unDredc(c);
@@ -771,13 +831,25 @@ fixDolMod(void)
 	unDredc(c);
 }
 
+static int
+all_dolmcnts_are_0(void)
+{
+    int i = 0;
+    for (; i < ndolflags; ++i) {
+	if (dolmcnts[i] != 0)
+	    return 0;
+    }
+    return 1;
+}
+
 static void
 setDolp(Char *cp)
 {
     Char *dp;
     size_t i;
+    int nthMod = 0;
 
-    if (dolmod.len == 0 || dolmcnt == 0) {
+    if (dolmod.len == 0 || all_dolmcnts_are_0()) {
 	dolp = cp;
 	return;
     }
@@ -786,10 +858,12 @@ setDolp(Char *cp)
 	int didmod = 0;
 
 	/* handle s// [eichin:19910926.0510EST] */
-	if(dolmod.s[i] == 's') {
+	if (dolmod.s[i] == 's') {
 	    Char delim;
 	    Char *lhsub, *rhsub, *np;
 	    size_t lhlen = 0, rhlen = 0;
+	    /* keep track of where the last :a match hit */
+	    ptrdiff_t last_match = 0;
 
 	    delim = dolmod.s[++i];
 	    if (!delim || letter(delim)
@@ -798,47 +872,50 @@ setDolp(Char *cp)
 		break;
 	    }
 	    lhsub = &dolmod.s[++i];
-	    while(dolmod.s[i] != delim && dolmod.s[++i]) {
+	    while (dolmod.s[i] != delim && dolmod.s[++i]) {
 		lhlen++;
 	    }
 	    dolmod.s[i] = 0;
 	    rhsub = &dolmod.s[++i];
-	    while(dolmod.s[i] != delim && dolmod.s[++i]) {
+	    while (dolmod.s[i] != delim && dolmod.s[++i]) {
 		rhlen++;
 	    }
 	    dolmod.s[i] = 0;
 
 	    strip(lhsub);
 	    strip(rhsub);
-	    strip(cp);
-	    dp = cp;
-	    do {
-		dp = Strstr(dp, lhsub);
-		if (dp) {
-		    ptrdiff_t diff = dp - cp;
-		    size_t len = (Strlen(cp) + 1 - lhlen + rhlen);
-		    np = xmalloc(len * sizeof(Char));
-		    (void) Strncpy(np, cp, diff);
-		    (void) Strcpy(np + diff, rhsub);
-		    (void) Strcpy(np + diff + rhlen, dp + lhlen);
+	    if (dolmcnts[nthMod] != 0) {
+	        strip(cp);
+	        dp = cp;
+	        do {
+	            dp = Strstr(dp + last_match, lhsub);
+	            if (dp) {
+	                ptrdiff_t diff = dp - cp;
+	                size_t len = (Strlen(cp) + 1 - lhlen + rhlen);
+	                np = xmalloc(len * sizeof(Char));
+	                (void) Strncpy(np, cp, diff);
+	                (void) Strcpy(np + diff, rhsub);
+	                (void) Strcpy(np + diff + rhlen, dp + lhlen);
+			last_match = diff + rhlen;
 
-		    xfree(cp);
-		    dp = cp = np;
-		    cp[--len] = '\0';
-		    didmod = 1;
-		    if (diff >= (ssize_t)len)
-			break;
-		} else {
-		    /* should this do a seterror? */
-		    break;
-		}
-	    }
-	    while (dol_flag_a != 0);
+	                xfree(cp);
+	                dp = cp = np;
+	                cp[--len] = '\0';
+	                didmod = 1;
+	                if (diff >= (ssize_t)len)
+	            	break;
+	            } else {
+	                /* should this do a seterror? */
+	                break;
+	            }
+	        }
+	        while (dolaflags[nthMod] != 0);
+            }
 	    /*
 	     * restore dolmod for additional words
 	     */
 	    dolmod.s[i] = rhsub[-1] = (Char) delim;
-        } else {
+	} else if (dolmcnts[nthMod] != 0) {
 
 	    do {
 		if ((dp = domod(cp, dolmod.s[i])) != NULL) {
@@ -856,14 +933,16 @@ setDolp(Char *cp)
 		else
 		    break;
 	    }
-	    while (dol_flag_a != 0);
+	    while (dolaflags[nthMod] != 0);
 	}
-	if (didmod && dolmcnt != INT_MAX)
-	    dolmcnt--;
+	if (didmod && dolmcnts[nthMod] != INT_MAX)
+	    dolmcnts[nthMod]--;
 #ifdef notdef
 	else
 	    break;
 #endif
+
+	++nthMod;
     }
 
     addla(cp);
@@ -921,7 +1000,7 @@ randsuf(void) {
 #ifndef WINNT_NATIVE
 	struct timeval tv;
 	(void) gettimeofday(&tv, NULL);
-	return putn((((tcsh_number_t)tv.tv_sec) ^ 
+	return putn((((tcsh_number_t)tv.tv_sec) ^
 	    ((tcsh_number_t)tv.tv_usec) ^
 	    ((tcsh_number_t)getpid())) & 0x00ffffff);
 #else
